@@ -3,6 +3,7 @@
 #include <gdk/gdkkeysyms.h>
 #include <stdlib.h>
 #include <string.h>
+#include <glib.h>
 
 #include "config.h"
 
@@ -10,6 +11,16 @@
 #include <libview/autoDrawer.h>
 #endif
 
+static gchar **args = NULL;
+static const GOptionEntry options [] =
+{
+  { 
+    G_OPTION_REMAINING, '\0', 0, G_OPTION_ARG_STRING_ARRAY, &args,
+    NULL, "hostname[:display]" },
+  { NULL, 0, 0, G_OPTION_ARG_NONE, NULL, NULL, 0 }
+};
+
+
 static GtkWidget *vnc;
 
 static void set_title(VncDisplay *vncdisplay, GtkWidget *window,
@@ -258,6 +269,8 @@ static gboolean window_state_event(GtkWidget *widget,
 
 int main(int argc, char **argv)
 {
+	GOptionContext *context;
+	GError *error = NULL;
 	char port[1024], hostname[1024];
 	char *display;
 	GtkWidget *window;
@@ -271,16 +284,28 @@ int main(int argc, char **argv)
 	GtkWidget *cab;
 	GtkWidget *fullscreen;
 	GtkWidget *scaling;
+	const char *help_msg = "Run 'gvncviewer --help' to see a full list of available command line options";
 
-	if (argc != 2 && argc != 3) {
-		fprintf(stderr, "Usage: %s hostname[:display] [password]\n",
-			argv[0]);
+	/* Setup command line options */
+	context = g_option_context_new ("- Simple VNC Client");
+	g_option_context_add_main_entries (context, options, NULL);
+	g_option_context_add_group (context, gtk_get_option_group (TRUE));
+	g_option_context_add_group (context, vnc_display_get_option_group ());
+	g_option_context_parse (context, &argc, &argv, &error);
+	if (error) {
+		g_print ("%s\n%s\n",
+			 error->message,
+			help_msg);
+		g_error_free (error);
+		return 1;
+	}
+	if (!args || (g_strv_length(args) != 1)) {
+		fprintf(stderr, "Usage: gvncviewer hostname[:display]\n%s\n", help_msg);
 		return 1;
 	}
 
-	gtk_init(&argc, &argv);
-
 	vnc = vnc_display_new();
+
 	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 #if WITH_LIBVIEW
 	layout = ViewAutoDrawer_New();
@@ -314,7 +339,7 @@ int main(int argc, char **argv)
 	submenu = gtk_menu_new();
 
 	fullscreen = gtk_check_menu_item_new_with_mnemonic("_Full Screen");
-	scaling = gtk_check_menu_item_new_with_mnemonic("OpenGL _Scaling");
+	scaling = gtk_check_menu_item_new_with_mnemonic("Scaled display");
 
 	gtk_menu_append(GTK_MENU(submenu), fullscreen);
 	gtk_menu_append(GTK_MENU(submenu), scaling);
@@ -332,10 +357,7 @@ int main(int argc, char **argv)
 	gtk_container_add(GTK_CONTAINER(window), layout);
 	gtk_widget_realize(vnc);
 
-	if (argc == 3)
-		vnc_display_set_credential(VNC_DISPLAY(vnc), VNC_DISPLAY_CREDENTIAL_PASSWORD, argv[2]);
-
-	snprintf(hostname, sizeof(hostname), "%s", argv[1]);
+	snprintf(hostname, sizeof(hostname), "%s", args[0]);
 	display = strchr(hostname, ':');
 
 	if (display) {
@@ -347,7 +369,11 @@ int main(int argc, char **argv)
 	vnc_display_open_host(VNC_DISPLAY(vnc), hostname, port);
 	vnc_display_set_keyboard_grab(VNC_DISPLAY(vnc), TRUE);
 	vnc_display_set_pointer_grab(VNC_DISPLAY(vnc), TRUE);
-	//vnc_display_set_pointer_local(VNC_DISPLAY(vnc), TRUE);
+
+	if (!gtk_widget_is_composited(window)) {
+		vnc_display_set_scaling(VNC_DISPLAY(vnc), TRUE);
+		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(scaling), TRUE);
+	}
 
 	gtk_signal_connect(GTK_OBJECT(window), "delete-event",
 			   GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
